tasc

Struct StdContext

source
pub struct StdContext { /* private fields */ }
The default context using crossbeam and the standard library.

This context facilitates creating new tasks and effectively dividing them among workers via a wait queue. Creating new threads uses the Rust standard library std::thread::spawn with its default settings.

Implementations

source

impl Context

source

pub fn new(handlers: usize) -> Self